(*
 * Evaluator for the source language.
 *
 * ----------------------------------------------------------------
 *
 * @begin[license]
 * Copyright (C) 2000 Jason Hickey, Caltech
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: Jason Hickey
 * @email{jyh@cs.caltech.edu}
 * @end[license]
 *)

open Symbol
open Fc_ast_type

(*
 * Type and variable environment.
 *)
type env

(*
 * Program values.
 *)
type evalue

(*
 * Evaluator exceptions.
 *)
type eval_exn =
   UnboundType of symbol
 | UnboundVar of symbol
 | UnboundField of symbol
 | NilPointer
 | TypeError of string
 | SubscriptOutOfBounds of int
 | DivideByZero
 | NotImplemented of string
 | NotAnInt
 | NotAnLValue
 | NotAScalar
 | NotAPointer
 | ArityMismatch

exception EvalException  of pos * eval_exn
exception Return         of pos * evalue
exception Break          of pos

(*
 * Starting environment.
 *)
val empty_env : env

(*
 * Evaluator.
 *)
val eval : env -> expr -> env * evalue
val eval_int : expr -> int
val eval_prog : env -> prog -> env
val eval_main : env -> string array -> unit

(*
 * Print a value.
 *)
val print_value : evalue -> unit

(*!
 * @docoff
 *
 * -*-
 * Local Variables:
 * Caml-master: "compile"
 * End:
 * -*-
 *)
